;=========================================================================
.model small, c
.386
include support.asm
.stack
;=========================================================================

ERROR_SUCCESS           equ 0
ERROR_INVALID_FUNCTION  equ 1
ERROR_INVALID_INPUT     equ 2
ERROR_INTERFACE         equ 3
ERROR_RESEND_COMMAND    equ 4
ERROR_NO_HANDLER        equ 5
ERROR_NOT_IMPLEMENTED1  equ 80h
ERROR_NOT_IMPLEMENTED2  equ 86h

;--------------------------------------------------------------------
; Sample rate is 10, 20, 40, 60, 80, 100, or 200 samples per second.
;--------------------------------------------------------------------

SAMPLE_RATE_10          equ 0
SAMPLE_RATE_20          equ 1
SAMPLE_RATE_40          equ 2
SAMPLE_RATE_60          equ 3
SAMPLE_RATE_80          equ 4
SAMPLE_RATE_100         equ 5
SAMPLE_RATE_200         equ 6

;-----------------------------------------
; Resolution 1, 2, 4, or 8 counts per mm.
;-----------------------------------------

RESOLUTION_1            equ 0
RESOLUTION_2            equ 1
RESOLUTION_3            equ 2
RESOLUTION_8            equ 3

;---------------------------------------------------------------------
; Structure to hold status values returned by ReturnStatus procedure.
;---------------------------------------------------------------------

_STATUS STRUCT
  statusByte db ?
  resolution db ?
  sampleRate db ?
_STATUS ENDS

;=========================================================================
.data
    status    _STATUS <>
    deviceID  db      0
.code
;=========================================================================
ShowErrorStatus proc npCaption:WORD, wStatus:WORD
    .if wStatus == ERROR_SUCCESS
        print npCaption,"ERROR_SUCCESS",NL
    .elseif wStatus == ERROR_INVALID_FUNCTION
        print npCaption,"ERROR_INVALID_FUNCTION",NL
    .elseif wStatus == ERROR_INVALID_INPUT
        print npCaption,"ERROR_INVALID_INPUT",NL
    .elseif wStatus == ERROR_INTERFACE
        print npCaption,"ERROR_INTERFACE",NL
    .elseif wStatus == ERROR_RESEND_COMMAND
        print npCaption,"ERROR_RESEND_COMMAND",NL
    .elseif wStatus == ERROR_NO_HANDLER
        print npCaption,"ERROR_NO_HANDLER",NL
    .elseif wStatus == ERROR_NOT_IMPLEMENTED1 || ERROR_NOT_IMPLEMENTED2
        print npCaption,"ERROR_NOT_IMPLEMENTED",NL
    .endif
    ret
ShowErrorStatus endp
;=========================================================================
DisableMouse proc uses bx
    mov ax, 0c200h
    xor bh, bh
    int 15h
    movzx ax, ah
    jc  @f
    xor ax, ax
  @@:
    ret
DisableMouse endp
;=========================================================================
EnableMouse proc uses bx
    mov ax, 0c200h
    mov bh, 1
    int 15h
    movzx ax, ah
    jc  @f
    xor ax, ax
  @@:
    ret
EnableMouse endp
;=========================================================================
ResetMouse proc
    mov ax, 0c201h
    int 15h
    movzx ax, ah
    jc  @f
    xor ax, ax
  @@:
    ret
ResetMouse endp
;=========================================================================
SetSampleRate proc uses bx sampleRate:WORD
    mov ax, 0c202h
    mov bx, sampleRate
    mov bh, bl
    int 15h
    movzx ax, ah
    jc  @f
    xor ax, ax
  @@:
    ret
SetSampleRate endp
;=========================================================================
SetResolution proc uses bx resolution:WORD
    mov ax, 0c203h
    mov bx, resolution
    mov bh, bl
    int 15h
    movzx ax, ah
    jc  @f
    xor ax, ax
  @@:
    ret
SetResolution endp
;=========================================================================
GetDeviceID proc uses bx di npID:WORD
    mov ax, 0c204h
    int 15h
    movzx ax, ah
    jc  @f
    mov di, npID
    mov [di], bh
    xor ax, ax
  @@:
    ret
GetDeviceID endp
;=========================================================================
InitializeMouse proc uses bx dataPackageSize:WORD
    mov ax, 0c205h
    mov bx, dataPackageSize
    mov bh, bl
    int 15h
    movzx ax, ah
    jc  @f
    xor ax, ax
  @@:
    ret
InitializeMouse endp
;=========================================================================
ReturnStatus proc uses di bx cx dx npStatus:WORD
    mov ax, 0c206h
    xor bh, bh
    int 15h
    movzx ax, ah
    jc  @f
    mov di, npStatus
    mov [di]._STATUS.statusByte, bl
    mov [di]._STATUS.resolution, cl
    mov [di]._STATUS.sampleRate, dl
    xor ax, ax
  @@:
    ret
ReturnStatus endp
;=========================================================================
SetHandler proc uses es bx handlerSeg:WORD, handlerOffset:WORD
    mov ax, 0c207h
    mov es, handlerSeg
    mov bx, handlerOffset
    int 15h
    movzx ax, ah
    jc  @f
    xor ax, ax
  @@:
    ret
SetHandler endp
;=========================================================================
handler:
    retf
;=========================================================================
.startup
;=========================================================================

    invoke ResetMouse
    invoke ShowErrorStatus, chr$("Reset : "), ax
    invoke GetDeviceID, addr deviceID
    invoke ShowErrorStatus, chr$("GetDeviceID : "), ax
    print byte$(deviceID),NL
    invoke DisableMouse
    invoke ShowErrorStatus, chr$("Disable : "), ax
    invoke SetHandler, seg handler, offset handler
    invoke ShowErrorStatus, chr$("SetHandler : "), ax
    print "Press any key to continue..."
    print addr crlf
    call WaitKey
    invoke EnableMouse
    invoke ShowErrorStatus, chr$("Enable : "), ax
    invoke ReturnStatus, addr status
    invoke ShowErrorStatus, chr$("ReturnStatus : "), ax
    print "statusByte : ",hexbyte$(status.statusByte),"h",NL
    print "resolution : ",byte$(status.resolution),NL
    print "sampleRate : ",byte$(status.sampleRate),NL

    call WaitKey
;=========================================================================
.exit
end

Handler proc far w3:WORD,w2:WORD,w1:WORD,w0:WORD
    ret
Handler endp
